#!/usr/bin/env python

"""Pick data-points with a cursor.

Reads in ASCII data from datafile, or, if datafile is not given, from
stdin.  Data files define an array of (x,y) points and consist of two
columns (x&y) seperated by TABs, with ENDLINEs between the points:

x0	y0
x1	y1
...

The points are plotted in a window, and mousing over the plot displays
a cursor that snaps to the nearest point.  Right-clicking (button 3)
will print the TAB seperated coordinates (x, y, point-index) to stdout.
"""

import matplotlib.pyplot as _pyplot
import numpy as _numpy


__version__ = '0.3'


class Picker (object):
    """Pick points from a plot
    """
    def __init__(self, axes, x=None, y=None, selected=None, highlight=False):
        self.axes = axes
        if highlight:
            if selected is None:
                selected = []
            sx = [x[i] for i in selected]
            sy = [y[i] for i in selected]
            hold = self.axes.ishold()
            axes.hold(True)
            self.highlight_line, = axes.plot(sx, sy, 'r.', zorder=5)
            self.axes.hold(hold)
        self.selected = selected
        self.highlight = highlight
        self.txt = self.axes.title

    def onpick(self, event):
        thisline = event.artist
        xdata = thisline.get_xdata()
        ydata = thisline.get_ydata()
        for i in event.ind:
            if self.highlight:
                if i in self.selected:
                    self.selected.remove(i)
                else:
                    self.selected.append(i)
                sx = [xdata[i] for i in self.selected]
                sy = [ydata[i] for i in self.selected]
                self.highlight_line.set_data(sx, sy)
                _pyplot.draw()
            else:
                print('{}\t{}\t{}'.format(xdata[i], ydata[i], i))
            break  # only deal with the first picked point


if __name__ == "__main__" :
    from argparse import ArgumentParser as _ArgumentParser
    import sys as _sys

    parser = _ArgumentParser(
        description=__doc__, version=__version__)
    parser.add_argument(
        '--highlight', action='store_const', const=True, default=False,
        help=('Highlight selected points and print them afterwards, instead '
              'of printing them as they are clicked'))
    parser.add_argument(
        'datafile', nargs='?', default=None,
        help='Path to the datafile.  Defaults to stdin')

    args = parser.parse_args()

    if args.datafile:
        data = _numpy.genfromtxt(args.datafile)
    else:
        data = _numpy.genfromtxt(_sys.stdin)
    x = data[:,0].squeeze()
    y = data[:,1].squeeze()

    figure = _pyplot.figure()
    axes = figure.add_subplot(1, 1, 1)
    axes.plot(x, y, '.', picker=2)
    axes.autoscale(tight=True)
    picker = Picker(axes=axes, highlight=args.highlight)
    figure.canvas.mpl_connect('pick_event', picker.onpick)

    print('#x\ty\tindex')
    _pyplot.show()
    if args.highlight:
        for i in picker.selected:
            point = (x[i], y[i], i)
            print('\t'.join(str(x) for x in point))
